package net.jaekl.frank.octranspo;

import java.util.ArrayList;
import net.jaekl.qd.xml.ParseResult;
import net.jaekl.qd.xml.XmlParseException;

public class Route extends ParseResult {
	static final String ROUTE = "Route";
	
	static final String ROUTE_NO = "RouteNo";
	static final String DIRECTION_ID = "DirectionID";
	static final String DIRECTION = "Direction";
	static final String ROUTE_HEADING = "RouteHeading";
	static final String ROUTE_LABEL = "RouteLabel";	// What GetNextTripsForStop calls <RouteHeading>	
	static final String TRIPS = "Trips";
	
	static final String TRIP = "Trip";
	
	static final String[] INTERNAL = { ROUTE_NO, DIRECTION_ID, DIRECTION, ROUTE_HEADING, ROUTE_LABEL, TRIPS };
	static final Object[][] EXTERNAL = { { TRIP, Trip.class } };
	
	int m_routeNo;
	int m_directionID;
	String m_direction;
	String m_routeHeading;
	ArrayList<Trip> m_trips;

	public Route() {
		super(ROUTE, INTERNAL, EXTERNAL);
		m_routeNo = 0;
		m_directionID = 0;
		m_direction = "";
		m_routeHeading = "";
		m_trips = new ArrayList<Trip>();
	}

	public int getRouteNo() { return m_routeNo; }
	public int getDirectionID() { return m_directionID; }
	public String getDirection() { return m_direction; }
	public String getRouteHeading() { return m_routeHeading; } 
	public int getNumTrips() { return m_trips.size(); }
	public Trip getTrip(int idx) { return new Trip(m_trips.get(idx)); }

	@Override
	public void endContents(String uri, String localName, String qName, String chars) throws XmlParseException 
	{
		if (ROUTE_NO.equals(localName)) {
			m_routeNo = Integer.parseInt(chars);
		}
		else if (DIRECTION_ID.equals(localName)) {
			m_directionID = Integer.parseInt(chars);
		}
		else if (DIRECTION.equals(localName)) {
			m_direction = chars;
		}
		else if (ROUTE_HEADING.equals(localName) || ROUTE_LABEL.equals(localName)) {
			m_routeHeading = chars;
		}
	}
	
	@Override
	public void endExternal(String uri, String localName, String qName) throws XmlParseException
	{
		if (localName.equals(TRIP)) {
			ParseResult[] collected = collectParsedChildren(Trip.class);
			for (ParseResult pr : collected) {
				assert(pr instanceof Trip);
				m_trips.add((Trip)pr);
			}
		}
	}
}
