package net.jaekl.frank.octranspo;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.ArrayList;

import net.jaekl.frank.FrankException;
import net.jaekl.qd.QDException;
import net.jaekl.qd.http.RequestBroker;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;

public class Server
{
	static final String API_KEY = "apiKey";
	static final String APP_ID = "appID";
	static final String GATEWAY_URL = "https://api.octranspo1.com/v1.2/";
	static final String ROUTE_NO = "routeNo";
	static final String STOP_NO = "stopNo";
	static final String GET_ROUTE_SUMMARY_FOR_STOP = "GetRouteSummaryForStop";
	static final String GET_NEXT_TRIPS_FOR_STOP = "GetNextTripsForStop";
	static final String GET_NEXT_TRIPS_FOR_STOP_ALL_ROUTES = "GetNextTripsForStopAllRoutes";

	RequestBroker m_broker;
	ArrayList<NameValuePair> m_baseParams;

	// Constructor
	// appID   The appID for our application
	// is      InputStream from which to read the apiKey
	public Server(String appID, InputStream is) throws IOException {
		assert (null != appID);
		assert (null != is);

		BufferedReader br = new BufferedReader(new InputStreamReader(is));
		String apiKey = br.readLine();

		m_baseParams = new ArrayList<NameValuePair>();
		m_baseParams.add(new BasicNameValuePair(APP_ID, appID));
		m_baseParams.add(new BasicNameValuePair(API_KEY, apiKey));

		m_broker = new RequestBroker(GATEWAY_URL, m_baseParams);
	}

	public StopInfo getRouteSummaryForStop(int stopNo) 
			throws FrankException
	{
		try {
			ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
			params.add(new BasicNameValuePair(STOP_NO, Integer.valueOf(stopNo).toString()));
			return (StopInfo) m_broker.submitAndParse(GET_ROUTE_SUMMARY_FOR_STOP, params, RouteSummary.class);
		}
		catch (QDException e) {
			throw new FrankException(e);
		}
	}

	public StopInfo getNextTripsForStop(int stopNo, int routeNo) 
			throws FrankException
	{
		try {
			ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
			params.add(new BasicNameValuePair(STOP_NO, Integer.valueOf(stopNo).toString()));
			params.add(new BasicNameValuePair(ROUTE_NO, Integer.valueOf(routeNo).toString()));
			return (StopInfo) m_broker.submitAndParse(GET_NEXT_TRIPS_FOR_STOP, params, NextTrips.class);
		}
		catch (QDException e) {
			throw new FrankException(e);
		}
	}

	public StopInfo getNextTripsForStopAllRoutes(int stopNo) 
			throws FrankException
	{
		try {
			ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
			params.add(new BasicNameValuePair(STOP_NO, Integer.valueOf(stopNo).toString()));
			return (StopInfo) m_broker.submitAndParse(GET_NEXT_TRIPS_FOR_STOP_ALL_ROUTES, params, RouteSummary.class);
		}
		catch (QDException e) {
			throw new FrankException(e);
		}
	}
}

