package net.jaekl.frank.octranspo;

import java.util.ArrayList;

import net.jaekl.qd.xml.ParseResult;
import net.jaekl.qd.xml.XmlParseException;

public class StopInfo extends ParseResult
{
	// potential child tag names
	public static String STOP_NO = "StopNo";
	public static String DESCRIPTION = "StopDescription";	// Present in RouteSummary
	public static String STOP_LABEL = "StopLabel";			// What NextTrips calls StopDescription
	public static String ERROR = "Error";
	public static String ROUTES = "Routes";
	public static String ROUTE = "Route";

	// data returned inside our element
	int m_stopNo;
	String m_descr;
	String m_error;
	ArrayList<Route> m_routes;

	// Constructor
	public StopInfo(String rootTagName, String[] internal, Object[][] external) {
		super(rootTagName, internal, external);

		m_stopNo = 0;
		m_descr = "";
		m_error = "";
		m_routes = new ArrayList<Route>();
	}

	// -----------------------------
	// Public methods to access data

	public int getStopNo() { return m_stopNo; }
	public String getDescr() { return m_descr; }
	public String getError() { return m_error; }
	public int getNumRoutes() { return m_routes.size(); }
	public Route getRoute(int idx) { return m_routes.get(idx); }

	// --------------------------
	// ParseResult implementation

	@Override
	public void endContents(String uri, String localName, String qName, String chars) throws XmlParseException
	{
		assert (null != localName);

		if (localName.equals(STOP_NO)) {
			m_stopNo = Integer.parseInt(chars);
		}
		else if (localName.equals(DESCRIPTION) || localName.equals(STOP_LABEL)) {
			m_descr = chars;
		}
		else if (localName.equals(ERROR)) {
			m_error = chars;
		}
	}
	
	@Override
	public void endExternal(String uri, String localName, String qName) throws XmlParseException
	{
		if (localName.equals(ROUTE)) {
			ParseResult[] collected = collectParsedChildren(Route.class);
			for (ParseResult pr : collected) {
				assert (pr instanceof Route);
				m_routes.add((Route)pr);
			}
		}
	}
	
	
}

