package net.jaekl.frank.octranspo;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import net.jaekl.qd.util.ParseUtils;
import net.jaekl.qd.xml.ParseResult;
import net.jaekl.qd.xml.XmlParseException;

public class Trip extends ParseResult {
	static final String TRIP = "Trip";
	
	static final String TRIP_DESTINATION = "TripDestination";
	static final String TRIP_START_TIME = "TripStartTime";
	static final String ADJUSTED_SCHEDULE_TIME = "AdjustedScheduleTime";
	static final String ADJUSTMENT_AGE = "AdjustmentAge";
	static final String LAST_TRIP_OF_SCHEDULE = "LastTripOfSchedule";
	static final String BUS_TYPE = "BusType";
	static final String GPS_SPEED = "GPSSpeed";
	static final String LATITUDE = "Latitude";
	static final String LONGITUDE = "Longitude";
	
	static final String[] INTERNAL = { TRIP_DESTINATION,
		                               TRIP_START_TIME,
		                               ADJUSTED_SCHEDULE_TIME,
		                               ADJUSTMENT_AGE,
		                               LAST_TRIP_OF_SCHEDULE,
		                               BUS_TYPE,
		                               GPS_SPEED,
		                               LATITUDE,
		                               LONGITUDE };
	static final Object[][] EXTERNAL = {};
	
    String  m_dest;      // destination
    Date    m_start;     // time at which the trip started / is scheduled to start
    int     m_adjTime;   // minutes until bus is predicted to arrive
    double  m_adjAge;    // time since the last GPS data was received, in minutes (possibly fractional)
    boolean m_lastTrip;  // is this the last scheduled trip of the day?
    String  m_busType;   // type of bus
    double  m_speed;     // speed (km/h) when last polled
    double  m_long;      // longitude
    double  m_lat;       // latitude
    
    DateFormat m_dateFormat;
    Date       m_constructed;	// DateTime when this object was constructed

    public Trip() {
    	super(TRIP, INTERNAL, EXTERNAL);
    	m_dest = "";
    	m_start = new Date();
    	m_adjTime = 0;
    	m_adjAge = 0.0;
    	m_lastTrip = false;
    	m_busType = "";
    	m_speed = 0.0;
    	m_long = 0.0;
    	m_lat = 0.0;
    	
    	m_dateFormat = new SimpleDateFormat("hh:mm");
    	m_constructed = new Date();
    }
    
    public Trip(Trip other) {
    	super(TRIP, INTERNAL, EXTERNAL);
    	m_dest = other.m_dest;
    	m_start = other.m_start;
    	m_adjTime = other.m_adjTime;
    	m_adjAge = other.m_adjAge;
    	m_lastTrip = other.m_lastTrip;
    	m_busType = other.m_busType;
    	m_speed = other.m_speed;
    	m_long = other.m_long;
    	m_lat = other.m_lat;
    	m_constructed = other.m_constructed;
	}

	public String getDest() { return m_dest; }
    public Date getStart() { return m_start; }
    public int getAdjTime() { return m_adjTime; }
    public double getAdjAge() { return m_adjAge; }
    public boolean isLastTrip() { return m_lastTrip; }
    public String getBusType() { return m_busType; }
    public double getSpeed() { return m_speed; }
    public double getLongitude() { return m_long; }
    public double getLatitude() { return m_lat; }
    
    // Estimated (Date)Time of Arrival of this trip at the stop 
    public Date getETA() { return new Date(m_constructed.getTime() + (long)(1000 * 60 * m_adjTime)); }
    
    // (Date)Time when the GPS for this bus was last read
    public Date getGPSTime() { return new Date(m_constructed.getTime() - (long)(1000 * 60 * m_adjAge)); } 

    // ---------------------------
    // ParseResult implementation:
    
	@Override
	public void endContents(String uri, String localName, String qName,	String chars) throws XmlParseException 
	{
		try {
			if (TRIP_DESTINATION.equals(localName)) {
				m_dest = chars;
			}
			else if (TRIP_START_TIME.equals(localName)) {
				m_start = m_dateFormat.parse(chars);
			}
			else if (ADJUSTED_SCHEDULE_TIME.equals(localName)) {
				m_adjTime = ParseUtils.parseInt(chars);
			}
			else if (ADJUSTMENT_AGE.equals(localName)) {
				m_adjAge = ParseUtils.parseDouble(chars);
			}
			else if (LAST_TRIP_OF_SCHEDULE.equals(localName)) {
				m_lastTrip = Boolean.parseBoolean(chars);
			}
			else if (BUS_TYPE.equals(localName)) {
				m_busType = chars;
			}
			else if (GPS_SPEED.equals(localName)) {
				m_speed = ParseUtils.parseDouble(chars);
			}
			else if (LONGITUDE.equals(localName)) {
				m_long = ParseUtils.parseDouble(chars);
			}
			else if (LATITUDE.equals(localName)) {
				m_lat = ParseUtils.parseDouble(chars);
			}
		} catch (ParseException pe) {
			throw new XmlParseException(pe);
		}
	}
	
	@Override
	public void endExternal(String uri, String localName, String qName) throws XmlParseException
	{
		// no externally-parsed children
	}
}

