// Copyright (C) 2004, 2014 Christian Jaekl

// Central spot from which to access ResourceBundles.
// This made more sense with earlier versions of Java, where the specification did not 
// guarantee that ResourceBundles would be cached.  Java 7 and later cache by default,
// but it still seems prudent to centralize accesses to resources here so that we have 
// control in case we want to implement our own cache, or override certain behaviours.
// 
// Note that we rely on the JVM's caching, to avoid unnecessary overhead.
// See http://java2go.blogspot.ca/2010/03/dont-be-smart-never-implement-resource.html

package net.jaekl.qd;

import java.util.Locale;
import java.util.ResourceBundle;

public class QDBundleFactory {
	static volatile QDBundleFactory m_inst;	// singleton instance
	
	private QDBundleFactory() {
		// no-op
	}
	
	public static QDBundleFactory getInst() {
		QDBundleFactory result = m_inst;
		if (null == result) {
			synchronized(QDBundleFactory.class) {
				if (null == m_inst) {
					m_inst = new QDBundleFactory();
				}
				result = m_inst;
			}
		}
		return result;
	}
	
	public ResourceBundle getBundle(String baseName, Locale locale) {
		return ResourceBundle.getBundle(baseName, locale);
	}
}
