package net.jaekl.frank.octranspo;

import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import junit.framework.Assert;
import net.jaekl.frank.FrankException;
import net.jaekl.qd.QDException;
import net.jaekl.qd.http.RequestBrokerMock;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.junit.Test;

public class ServerTest {
	private static final String ROUTE_SUMMARY_FOR_STOP = 
			"<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
					+ "<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" 
					+  "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"\n" 
					+  "xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\n"
					+ "	<soap:Body>\n"
					+ "		<GetRouteSummaryForStopResponse xmlns=\"http://octranspo.com\">\n"
					+ "			<GetRouteSummaryForStopResult>\n"
					+ "				<StopNo xmlns=\"http://tempuri.org/\">1234</StopNo>\n"
					+ "				<StopDescription" 
					+ "				 xmlns=\"http://tempuri.org/\">ONE-TWO-THREE-FOUR</StopDescription>\n"
					+ "				<Error xmlns=\"http://tempuri.org/\"/>\n"
					+ "				<Routes xmlns=\"http://tempuri.org/\">\n"
					+ "					<Route>\n"
					+ "						<RouteNo>123</RouteNo>\n"
					+ "						<DirectionID>0</DirectionID>\n"
					+ "						<Direction>NORTH</Direction>\n"
					+ "						<RouteHeading>First Mall</RouteHeading>\n"
					+ "					</Route>\n"
					+ "					<Route>\n"
					+ "						<RouteNo>123</RouteNo>\n"
					+ "						<DirectionID>1</DirectionID>\n"
					+ "						<Direction>SOUTH</Direction>\n"
					+ "						<RouteHeading>Second Mall</RouteHeading>\n"
					+ "					</Route>\n"
					+ "				</Routes>\n"
					+ "			</GetRouteSummaryForStopResult>\n"
					+ "		</GetRouteSummaryForStopResponse>\n"
					+ "	</soap:Body>\n"
					+ "</soap:Envelope>\n";

	private static final String NEXT_TRIPS_FOR_STOP =  
			"<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
					"<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">" +
					"<soap:Body>" +
					"<GetNextTripsForStopResponse xmlns=\"http://octranspo.com\">" +
					"<GetNextTripsForStopResult>" +
					"<StopNo xmlns=\"http://tempuri.org/\">7659</StopNo>" +
					"<StopLabel xmlns=\"http://tempuri.org/\">BANK FIFTH</StopLabel>" +
					"<Error xmlns=\"http://tempuri.org/\" />" +
					"<Route xmlns=\"http://tempuri.org/\">" +
					"<RouteDirection>" +
					"<RouteNo>1</RouteNo><RouteLabel>Ottawa-Rockcliffe</RouteLabel>" +
					"<Direction>Northbound</Direction><Error /><RequestProcessingTime>20141130111450</RequestProcessingTime>" +
					"<Trips>" +
					"<Trip>" +
					"<TripDestination>Ottawa Rockcliffe</TripDestination>" +
					"<TripStartTime>11:10</TripStartTime>" +
					"<AdjustedScheduleTime>20</AdjustedScheduleTime>" +
					"<AdjustmentAge>0.18</AdjustmentAge>" +
					"<LastTripOfSchedule>false</LastTripOfSchedule>" +
					"<BusType>4LA - DEH</BusType>" +
					"<Latitude>45.352026</Latitude><Longitude>-75.649993</Longitude><GPSSpeed>50.0</GPSSpeed>" +
					"</Trip>" +
					"<Trip>" +
					"<TripDestination>Ottawa Rockcliffe</TripDestination>" +
					"<TripStartTime>11:40</TripStartTime>" +
					"<AdjustedScheduleTime>51</AdjustedScheduleTime><AdjustmentAge>-1</AdjustmentAge>" +
					"<LastTripOfSchedule>false</LastTripOfSchedule><BusType>4E - DEH</BusType>" +
					"<Latitude /><Longitude /><GPSSpeed />" +
					"</Trip>" +
					"<Trip>" +
					"<TripDestination>Ottawa Rockcliffe</TripDestination>" +
					"<TripStartTime>12:00</TripStartTime>" +
					"<AdjustedScheduleTime>71</AdjustedScheduleTime><AdjustmentAge>-1</AdjustmentAge>" +
					"<LastTripOfSchedule>false</LastTripOfSchedule><BusType>4E - DEH</BusType>" +
					"<Latitude /><Longitude /><GPSSpeed />" +
					"</Trip></Trips>" +
					"</RouteDirection>" +
					"</Route>" +
					"</GetNextTripsForStopResult>" +
					"</GetNextTripsForStopResponse>" +
					"</soap:Body>" +
					"</soap:Envelope>";	
	
	private static final String NEXT_TRIPS_FOR_STOP_ALL_ROUTES = 
			"<?xml version=\"1.0\" encoding=\"utf-8\"?>" + 
					"<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">" +
					"<soap:Body>" +
					"<GetRouteSummaryForStopResponse xmlns=\"http://octranspo.com\">" +
					"<GetRouteSummaryForStopResult>" +
					"<StopNo xmlns=\"http://tempuri.org/\">7659</StopNo>" +
					"<StopDescription xmlns=\"http://tempuri.org/\">BANK FIFTH</StopDescription>" +
					"<Error xmlns=\"http://tempuri.org/\"/>" +
					"<Routes xmlns=\"http://tempuri.org/\">" +
					"<Route>" +
					"<RouteNo>1</RouteNo><DirectionID>1</DirectionID><Direction>Northbound</Direction>" +
					"<RouteHeading>Ottawa-Rockcliffe</RouteHeading>" +
					"<Trips>" +
					"<Trip>" +
					"<TripDestination>Ottawa Rockcliffe</TripDestination><TripStartTime>11:40</TripStartTime>" +
					"<AdjustedScheduleTime>2</AdjustedScheduleTime><AdjustmentAge>0.55</AdjustmentAge>" +
					"<LastTripOfSchedule/><BusType>4E - DEH</BusType>" +
					"<Latitude>45.392965</Latitude><Longitude>-75.682561</Longitude><GPSSpeed>35.7</GPSSpeed>" +
					"</Trip>" +
					"<Trip>" +
					"<TripDestination>Ottawa Rockcliffe</TripDestination><TripStartTime>12:00</TripStartTime>" +
					"<AdjustedScheduleTime>17</AdjustedScheduleTime><AdjustmentAge>0.62</AdjustmentAge>" +
					"<LastTripOfSchedule/><BusType>4E - DEH</BusType>" +
					"<Latitude>45.362237</Latitude><Longitude>-75.658112</Longitude><GPSSpeed>57.6</GPSSpeed>" +
					"</Trip>" +
					"<Trip>" +
					"<TripDestination>Ottawa Rockcliffe</TripDestination><TripStartTime>12:20</TripStartTime>" +
					"<AdjustedScheduleTime>37</AdjustedScheduleTime><AdjustmentAge>-1</AdjustmentAge>" +
					"<LastTripOfSchedule/><BusType>4E - DEH</BusType>" +
					"<Latitude/><Longitude/><GPSSpeed/>" +
					"</Trip>" +
					"</Trips>" +
					"</Route>" +
					"<Route>" +
					"<RouteNo>7</RouteNo><DirectionID>1</DirectionID><Direction>Eastbound</Direction>" +
					"<RouteHeading>St-Laurent</RouteHeading>" +
					"<Trips><Trip><TripDestination>St. Laurent</TripDestination><TripStartTime>12:11</TripStartTime>" +
					"<AdjustedScheduleTime>12</AdjustedScheduleTime><AdjustmentAge>0.61</AdjustmentAge>" +
					"<LastTripOfSchedule/><BusType>4LB - IN</BusType>" +
					"<Latitude>45.384370</Latitude><Longitude>-75.696361</Longitude><GPSSpeed>0.5</GPSSpeed>" +
					"</Trip>" +
					"<Trip>" +
					"<TripDestination>St. Laurent</TripDestination><TripStartTime>12:31</TripStartTime>" +
					"<AdjustedScheduleTime>32</AdjustedScheduleTime><AdjustmentAge>-1</AdjustmentAge>" +
					"<LastTripOfSchedule/><BusType>4L - DEH</BusType><Latitude/><Longitude/><GPSSpeed/>" +
					"</Trip>" +
					"<Trip>" +
					"<TripDestination>St. Laurent</TripDestination><TripStartTime>12:51</TripStartTime>" +
					"<AdjustedScheduleTime>52</AdjustedScheduleTime><AdjustmentAge>-1</AdjustmentAge>" +
					"<LastTripOfSchedule/><BusType>4LA - DEH</BusType>" +
					"<Latitude/><Longitude/><GPSSpeed/>" +
					"</Trip>" +
					"</Trips>" +
					"</Route>" +
					"</Routes>" +
					"</GetRouteSummaryForStopResult>" +
					"</GetRouteSummaryForStopResponse>" +
					"</soap:Body>" +
					"</soap:Envelope>";
	
	@Test
	public void testGetRouteSummaryForStop() throws IOException, QDException, FrankException {
		ServerMock sm = new ServerMock();
		RequestBrokerMock broker = sm.mock_getBroker();

		ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
		params.add(new BasicNameValuePair(Server.STOP_NO, "1234"));
		
		broker.setResult(Server.GET_ROUTE_SUMMARY_FOR_STOP, params, ROUTE_SUMMARY_FOR_STOP);
		
		StopInfo stopInfo = sm.getRouteSummaryForStop(1234);
		
		Assert.assertNotNull(stopInfo);
		Assert.assertEquals(1234, stopInfo.getStopNo());
		Assert.assertEquals("ONE-TWO-THREE-FOUR", stopInfo.getDescr());
		Assert.assertEquals("", stopInfo.getError());
		Assert.assertEquals(2, stopInfo.getNumRoutes());
		
		Route route;
		
		route = stopInfo.getRoute(0);
		Assert.assertEquals(123, route.getRouteNo());
		Assert.assertEquals(0, route.getDirectionID());
		Assert.assertEquals("NORTH", route.getDirection());
		Assert.assertEquals("First Mall", route.getRouteHeading());
		
		route = stopInfo.getRoute(1);
		Assert.assertEquals(123, route.getRouteNo());
		Assert.assertEquals(1, route.getDirectionID());
		Assert.assertEquals("SOUTH", route.getDirection());
		Assert.assertEquals("Second Mall", route.getRouteHeading());
	}

	@Test
	public void testGetNextTripsForStop() throws FrankException, IOException, ParseException {
		Route route;
		Trip trip;
		Date expectedDate; 
		DateFormat hourMinFormat = new SimpleDateFormat("hh:mm");
		
		ServerMock sm = new ServerMock();
		RequestBrokerMock broker = sm.mock_getBroker();

		ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
		params.add(new BasicNameValuePair(Server.STOP_NO, "7659"));
		params.add(new BasicNameValuePair(Server.ROUTE_NO, "1"));
		
		broker.setResult(Server.GET_NEXT_TRIPS_FOR_STOP, params, NEXT_TRIPS_FOR_STOP);
		
		StopInfo stopInfo = sm.getNextTripsForStop(7659, 1);
		
		Assert.assertNotNull(stopInfo);
		Assert.assertEquals(7659, stopInfo.getStopNo());
		Assert.assertEquals("BANK FIFTH", stopInfo.getDescr());
		Assert.assertEquals("", stopInfo.getError());
		Assert.assertEquals(1, stopInfo.getNumRoutes());
		
		route = stopInfo.getRoute(0);
		Assert.assertEquals(1, route.getRouteNo());
		Assert.assertEquals("Northbound", route.getDirection());
		Assert.assertEquals("Ottawa-Rockcliffe", route.getRouteHeading());
		Assert.assertEquals(3, route.getNumTrips());
		
		trip = route.getTrip(0);
		Assert.assertEquals("Ottawa Rockcliffe", trip.getDest());
		expectedDate = hourMinFormat.parse("11:10");
		Assert.assertEquals(expectedDate, trip.getStart());
		Assert.assertEquals(20, trip.getAdjTime());
		Assert.assertEquals(0.18, trip.getAdjAge());
		Assert.assertEquals(false, trip.isLastTrip());
		Assert.assertEquals("4LA - DEH", trip.getBusType());
		Assert.assertEquals(50.0, trip.getSpeed());
		Assert.assertEquals(45.352026, trip.getLatitude());
		Assert.assertEquals(-75.649993, trip.getLongitude());
		
		trip = route.getTrip(2);
		Assert.assertEquals("Ottawa Rockcliffe", trip.getDest());
		expectedDate = hourMinFormat.parse("12:00");
		Assert.assertEquals(expectedDate, trip.getStart());
		Assert.assertEquals(71, trip.getAdjTime());
		Assert.assertEquals(-1.0, trip.getAdjAge());
		Assert.assertEquals(false, trip.isLastTrip());
		Assert.assertEquals("4E - DEH", trip.getBusType());
		Assert.assertEquals(0.0, trip.getSpeed());
		Assert.assertEquals(0.0, trip.getLatitude());
		Assert.assertEquals(0.0, trip.getLongitude());
	}

	@Test
	public void testGetNextTripsForStopAllRoutes() throws IOException, FrankException, ParseException {
		Route route;
		Trip trip;
		Date expectedDate; 
		DateFormat hourMinFormat = new SimpleDateFormat("hh:mm");
		
		ServerMock sm = new ServerMock();
		RequestBrokerMock broker = sm.mock_getBroker();

		ArrayList<NameValuePair> params = new ArrayList<NameValuePair>();
		params.add(new BasicNameValuePair(Server.STOP_NO, "7659"));
		
		broker.setResult(Server.GET_NEXT_TRIPS_FOR_STOP_ALL_ROUTES, params, NEXT_TRIPS_FOR_STOP_ALL_ROUTES);
		
		StopInfo stopInfo = sm.getNextTripsForStopAllRoutes(7659);
		
		Assert.assertNotNull(stopInfo);
		Assert.assertEquals(7659, stopInfo.getStopNo());
		Assert.assertEquals("BANK FIFTH", stopInfo.getDescr());
		Assert.assertEquals("", stopInfo.getError());
		Assert.assertEquals(2, stopInfo.getNumRoutes());
		
		route = stopInfo.getRoute(0);
		Assert.assertEquals(1, route.getRouteNo());
		Assert.assertEquals("Northbound", route.getDirection());
		Assert.assertEquals("Ottawa-Rockcliffe", route.getRouteHeading());
		Assert.assertEquals(3, route.getNumTrips());
		
		trip = route.getTrip(0);
		Assert.assertEquals("Ottawa Rockcliffe", trip.getDest());
		expectedDate = hourMinFormat.parse("11:40");
		Assert.assertEquals(expectedDate, trip.getStart());
		Assert.assertEquals(2, trip.getAdjTime());
		Assert.assertEquals(0.55, trip.getAdjAge());
		Assert.assertEquals(false, trip.isLastTrip());
		Assert.assertEquals("4E - DEH", trip.getBusType());
		Assert.assertEquals(35.7, trip.getSpeed());
		Assert.assertEquals(45.392965, trip.getLatitude());
		Assert.assertEquals(-75.682561, trip.getLongitude());
		
		trip = route.getTrip(2);
		Assert.assertEquals("Ottawa Rockcliffe", trip.getDest());
		expectedDate = hourMinFormat.parse("12:20");
		Assert.assertEquals(expectedDate, trip.getStart());
		Assert.assertEquals(37, trip.getAdjTime());
		Assert.assertEquals(-1.0, trip.getAdjAge());
		Assert.assertEquals(false, trip.isLastTrip());
		Assert.assertEquals("4E - DEH", trip.getBusType());
		Assert.assertEquals(0.0, trip.getSpeed());
		Assert.assertEquals(0.0, trip.getLatitude());
		Assert.assertEquals(0.0, trip.getLongitude());
	}

}
