package net.jaekl.qd.http;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;

import net.jaekl.qd.QDException;
import net.jaekl.qd.util.StringUtils;

import org.apache.http.NameValuePair;
import org.junit.Assert;

public class RequestBrokerMock extends RequestBroker {
	private static class InputParams {
		private String m_method;
		private ArrayList<NameValuePair> m_passedParams;
		
		public InputParams(String method, ArrayList<NameValuePair> passedParams) {
			m_method = method;
			m_passedParams = new ArrayList<NameValuePair>();
			for (NameValuePair nvp : passedParams) {
				m_passedParams.add(nvp);
			}
		}
		
		@Override
		public boolean equals(Object obj) {
			if (null == obj || !(obj instanceof InputParams)) {
				return false;
			}
			InputParams other = (InputParams)obj;
			if (! StringUtils.areEqual(m_method, other.m_method)) {
				return false;
			}
			if (m_passedParams.size() != other.m_passedParams.size()) {
				return false;
			}
			for (int i = 0; i < m_passedParams.size(); ++i) {
				NameValuePair a, b;
				a = m_passedParams.get(i);
				b = other.m_passedParams.get(i);
				if ( ! a.equals(b) ) {
					return false;
				}
			}
			return true;
		}
		
		@Override
		public int hashCode() {
			final int PRIME = 31;
			int result = PRIME + m_method.hashCode();
			for (NameValuePair nvp : m_passedParams) {
				result *= (PRIME + nvp.hashCode());
			}
			return result;
		}
	}
	
	HashMap<InputParams, Object> m_result;
	
	public RequestBrokerMock(String gatewayUrl, ArrayList<NameValuePair> baseParams) {
		super(gatewayUrl, baseParams);
		m_result = new HashMap<InputParams, Object>();
	}
	
	public void setResult(String method, ArrayList<NameValuePair> passedParams, Object result) {
		InputParams ip = new InputParams(method, passedParams);
		m_result.put(ip, result);
	}
	
	@Override 
	InputStream doSubmit(String method, ArrayList<NameValuePair> passedParams) throws QDException
	{
		InputParams ip = new InputParams(method, passedParams);
		Object result = m_result.get(ip);
		
		if (null == result) {
			Assert.fail("No result specified for given inputs.");
		}
		
		if (result instanceof QDException) {
			throw (QDException)result;
		}
		
		Assert.assertTrue(result instanceof String);
		
		InputStream is = null;
		
		try {
			String output = (String)result;
			is = new ByteArrayInputStream(output.getBytes("UTF-8"));
		}
		catch (UnsupportedEncodingException uee) {
			throw new QDException(uee);
		}
		
		return is;
	}
}
