// Copyright (C) 2004, 2014 Christian Jaekl

package net.jaekl.qd.util;

import java.io.Closeable;
import java.io.IOException;

import junit.framework.Assert;
import net.jaekl.qd.QDException;

import org.junit.Test;

public class ExceptionUtilsTest {

	private static class CloseableMock implements Closeable 
	{
		private IOException m_exception;
		private int m_timesClosed;
		
		public CloseableMock() {
			m_exception = null;
			m_timesClosed = 0;
		}
		
		public int getTimesClosed() { return m_timesClosed; }
		public void setExceptionToThrow(IOException ioe) { m_exception = ioe; }

		@Override
		public void close() throws IOException { 
			m_timesClosed++;
			if (null != m_exception) {
				throw new IOException("This is a test");
			}
		}
	}
	
	// Try closing a closeable.
	// It should be closed (once only).
	@Test
	public void test_tryClose_success() throws QDException
	{
		CloseableMock cm = new CloseableMock();
		
		ExceptionUtils.tryClose(cm);
		Assert.assertEquals(1, cm.getTimesClosed());
	}
	
	// Try closing a closeable that throws an IOException
	@Test
	public void test_tryClose_exception() throws QDException
	{
		final String MESSAGE = "This is a test";
		
		CloseableMock cm = new CloseableMock();
		cm.setExceptionToThrow(new IOException(MESSAGE));
		
		try {
			ExceptionUtils.tryClose(cm);
		}
		catch (QDException qde) {
			Assert.assertNotNull(qde);
			Throwable t = qde.getCause();
			Assert.assertNotNull(t);
			Assert.assertTrue(t instanceof IOException);
			Assert.assertEquals(MESSAGE, t.getMessage());
		}
	}
	
	// Try closing a null pointer. 
	// This should be harmless (no exception thrown).
	@Test
	public void test_tryClose_null() throws QDException 
	{
		ExceptionUtils.tryClose(null);
	}

}
